Unit STI_SONG;
{$D-}
interface

Uses Crt98;

Procedure STI_Play_Initialise;
Procedure STI_PlayString(ThePlayString : string );

implementation

{---------------------------------------------------------------------------}

Type
  PlayStr = String;

Var
  PlayString   : String;
  Note_Octave   : Integer;
  Note_Fraction : Real;
  Note_Duration : Integer;
  Note_Length   : Real;
  Note_Quarter  : Real;

{---------------------------------------------------------------------------}

Procedure STI_Play_Initialise;              {reset the play system          }

begin
   Note_Octave   := 4;                      {the default octave             }
   Note_Fraction := 0.875;                  {the default note fraction      }
   Note_Length   := 0.25;                   {the default note length        }
   Note_Quarter  := 500.0;                  {quarter of note period         }
end;

{---------------------------------------------------------------------------}

Procedure STI_PlayString(ThePlayString : PlayStr );

Const
   Note_Offset   : Array[ 'A'..'G' ] Of Integer
                   = ( 9, 11, 0, 2, 4, 5, 7 );
   Note_Freqs: Array[ 0 .. 84 ] Of Integer
               =
{
      C    C#     D    D#     E     F    F#     G    G#     A    A#     B
}
(     0,
     65,   69,   73,   78,   82,   87,   92,   98,  104,  110,  116,  123,
    131,  139,  147,  156,  165,  175,  185,  196,  208,  220,  233,  247,
    262,  278,  294,  312,  330,  350,  370,  392,  416,  440,  466,  494,
    524,  556,  588,  624,  660,  700,  740,  784,  832,  880,  932,  988,
   1048, 1112, 1176, 1248, 1320, 1400, 1480, 1568, 1664, 1760, 1864, 1976,
   2096, 2224, 2352, 2496, 2640, 2800, 2960, 3136, 3328, 3520, 3728, 3952,
   4192, 4448, 4704, 4992, 5280, 5600, 5920, 6272, 6656, 7040, 7456, 7904  );

   Quarter_Note = 0.25;

Var
   Play_Freq     : Integer;
   Play_Duration : Integer;
   Rest_Duration : Integer;
   I             : Integer;
   C             : Char;
   Freq          : Array[ 0 .. 6 , 0 .. 11 ] Of Integer ABSOLUTE Note_Freqs;
   N             : Integer;
   XN            : Real;
   K             : Integer;

{---------------------------------------------------------------------------}

Function GetInt : Integer;

Var
   N : Integer;

Begin
   N := 0;
   While( ThePlayString[I] In ['0'..'9'] ) Do
      Begin
         N := N * 10 + ORD(ThePlayString[I] ) - ORD('0');
         I := I + 1;
      End;
   I      := I - 1;
   GetInt := N;
End;

{---------------------------------------------------------------------------}

Begin (* PibPlay *)
   ThePlayString := ThePlayString+ ' ';
   I := 1;
   While( I < LENGTH(ThePlayString) ) Do
      Begin (* Interpret Music *)
         C := Upcase(ThePlayString[I]);
         Case C Of
            'A'..'G' : Begin (* A Note *)
                          N         := Note_Offset[ C ];
                          Play_Freq := Freq[ Note_Octave , N ];
                          XN := Note_Quarter * ( Note_Length / Quarter_Note );
                          Play_Duration := Trunc( XN * Note_Fraction );
                          Rest_Duration := Trunc( XN * ( 1.0 - Note_Fraction ) );
                          If ThePlayString[I+1] In ['#','+','-' ] Then
                             Begin
                                I := I + 1;
                                Case ThePlayString[I] OF
                                   '#' : Play_Freq :=
                                            Freq[ Note_Octave , N + 1 ];
                                   '+' : Play_Freq :=
                                            Freq[ Note_Octave , N + 1 ];
                                   '-' : Play_Freq :=
                                            Freq[ Note_Octave , N - 1 ];
                                End (* Case *);
                             End;
                          If ThePlayString[I+1] In ['0'..'9'] Then
                             Begin
                                I  := I + 1;
                                N  := GetInt;
                                XN := ( 1.0 / N ) / Quarter_Note;
                                Play_Duration :=
                                    Trunc( Note_Fraction * Note_Quarter * XN );
                                Rest_Duration :=
                                   Trunc( ( 1.0 - Note_Fraction ) *
                                          Xn * Note_Quarter );
                             End;
                             If ThePlayString[I+1] = '.' Then
                                Begin
                                   XN := 1.0;
                                   While( ThePlayString[I+1] = '.' ) Do
                                      Begin
                                         XN := XN * 1.5;
                                         I  := I + 1;
                                      End;
                                   Play_Duration :=
                                       Trunc( Play_Duration * XN );
                                End;
                          Sound( Play_Freq );
                          Delay( Play_Duration );
                          NoSound;
                          Delay( Rest_Duration );
                       End   (* A Note *);
            'M'      : Begin (* 'M' Commands *)
                          I := I + 1;
                          C := ThePlayString[I];
                          Case C Of
                             'F' : ;
                             'B' : ;
                             'N' : Note_Fraction := 0.875;
                             'L' : Note_Fraction := 1.000;
                             'S' : Note_Fraction := 0.750;
                             Else ;
                          End (* Case *);
                       End   (* 'M' Commands *);
            'O'      : Begin (* Set Octave *)
                          I := I + 1;
                          N := ORD(ThePlayString[I] ) - ORD('0');
                          If ( N < 0 ) OR ( N > 6 ) Then N := 4;
                          Note_Octave := N;
                       End   (* Set Octave *);
            '<'      : Begin (* Drop an octave *)
                          If Note_Octave > 0 Then
                             Note_Octave := Note_Octave - 1;
                       End   (* Drop an octave *);
            '>'      : Begin (* Ascend an octave *)
                          If Note_Octave < 6 Then
                             Note_Octave := Note_Octave + 1;
                       End   (* Ascend an octave *);
            'N'      : Begin (* Play Note N *)
                          I := I + 1;
                          N := GetInt;
                          If ( N > 0 ) AND ( N <= 84 ) Then
                             Begin
                                Play_Freq    := Note_Freqs[ N ];
                                XN           := Note_Quarter *
                                                ( Note_Length / Quarter_Note );
                                Play_Duration := Trunc( XN * Note_Fraction );
                                Rest_Duration := Trunc( XN * ( 1.0 - Note_Fraction ) );
                             End
                          Else If ( N = 0 ) Then
                             Begin
                                Play_Freq     := 0;
                                Play_Duration := 0;
                                Rest_Duration :=
                                   Trunc( Note_Fraction * Note_Quarter *
                                          ( Note_Length / Quarter_Note ) );
                             End;
                          Sound( Play_Freq );
                          Delay( Play_Duration );
                          NoSound;
                          Delay( Rest_Duration );
                       End   (* Play Note N *);
            'L'      : Begin (* Set Length of Notes *)
                          I := I + 1;
                          N := GetInt;
                          If N > 0 Then Note_Length := 1.0 / N;
                       End   (* Set Length of Notes *);
            'T'      : Begin (* # of quarter notes in a minute *)
                          I := I + 1;
                          N := GetInt;
                          Note_Quarter := ( 1092.0 / 18.2 / N ) * 1000.0;
                       End   (* # of quarter notes in a minute *);
            'P'      : Begin (* Pause *)
                          I := I + 1;
                          N := GetInt;
                          If ( N < 1 ) Then N := 1
                          Else If ( N > 64 ) Then N := 64;
                          Play_Freq     := 0;
                          Play_Duration := 0;
                          Rest_Duration :=
                             Trunc( ( ( 1.0 / N ) / Quarter_Note )
                                    * Note_Quarter );
                        {  Sound(Play_Freq);}
                          Delay( Play_Duration );
                          NoSound;
                          Delay( Rest_Duration );
                       End   (* Pause *);
         End;
         I := I + 1;
       End  (* Interpret Music *);
   NoSound;
   NoSound;
End;

{---------------------------------------------------------------------------}

Begin
  STI_Play_Initialise;
End.